'use strict';

var path = require('path');
var fs = require('fs');
var url = require('url');
var Koa = require('koa');
var serve = require('koa-static');
var mount = require('koa-mount');
var send = require('koa-send');
var router = require('koa-route');
var proxy = require('koa-better-http-proxy');
var color = require('colorette');
var ms = require('ms');
var dateTime = require('date-time');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Koa__default = /*#__PURE__*/_interopDefaultLegacy(Koa);
var serve__default = /*#__PURE__*/_interopDefaultLegacy(serve);
var mount__default = /*#__PURE__*/_interopDefaultLegacy(mount);
var send__default = /*#__PURE__*/_interopDefaultLegacy(send);
var router__default = /*#__PURE__*/_interopDefaultLegacy(router);
var proxy__default = /*#__PURE__*/_interopDefaultLegacy(proxy);
var color__default = /*#__PURE__*/_interopDefaultLegacy(color);
var ms__default = /*#__PURE__*/_interopDefaultLegacy(ms);
var dateTime__default = /*#__PURE__*/_interopDefaultLegacy(dateTime);

let app = null;
let quiet = false;
let veryQuiet = false;
let _info, _error;
const colorCodes = { 5: 'red', 4: 'yellow', 3: 'cyan', 2: 'green' };
const stamp = () => `[${dateTime__default['default']()}]`;
const header = color.blue('⚡︎dev-server');
const info = (...args) => !quiet && _info(...args);
const notice = (...args) => !veryQuiet && _info(header, ...args);
const setupStatic = (app, { basePath }) => (path) => app.use(mount__default['default'](basePath || '/', serve__default['default'](path)));
const setupProxy = (app) => ([src, dest]) => app.use(router__default['default'].all(src, proxy__default['default'](...(Array.isArray(dest) ? dest : [dest]))));
const logger = async (ctx, next) => {
  const start = Date.now();
  try {
    await next();
  } catch (err) { _error(err); }
  const status = ctx.status / 100 | 0;
  const c = color__default['default'][colorCodes[status] || 'reset'];
  info(stamp(), c(color.bold(ctx.method)), ctx.originalUrl, c('•'), color.dim(ctx.status), color.dim(ms__default['default'](Date.now() - start)));
};
function setupFallback (app, opts) {
  const fallbackFile = typeof opts.spa === 'boolean' ? 'index.html' : opts.spa;
  if (!fs.existsSync(fallbackFile)) return _error(header, color.red('Fallback file'), color.red(color.bold(path.resolve(fallbackFile))), color.red('from the SPA option not found - not setting up fallback'))
  else notice("using fallback file", color.bold(path.resolve(fallbackFile)));
  const fallback = async (ctx) => {
    if (ctx.accepts('html')) {
      info(stamp(), color.dim(`Serving ${fallbackFile} for`), ctx.originalUrl);
      await send__default['default'](ctx, fallbackFile);
    }
  };
  app.use(router__default['default'].get('*', fallback));
}
const printListenInfo = (server) => {
  const address = server.address();
  address.hostname = address.address;
  const a = url.format(address);
  notice("listening on", color.bold(color.green(`http://${a}`)));
};

async function initApp(opts = {}, testing) {
  quiet = !!opts.silent;
  veryQuiet = opts.silent === 'very';
  ({ info: _info, error: _error } = console);
  const warn = (!quiet && this && this.warn) || (() => {});
  if (!process.env.ROLLUP_WATCH && !opts.force) return warn(header + " cowardly refusing to start without 'watch' mode in rollup or 'force' option set")
  app = new Koa__default['default']();
  if (!quiet) app.use(logger);
  if (opts.extend) opts.extend(app, { router: router__default['default'], proxy: proxy__default['default'], send: send__default['default'], serve: serve__default['default'], mount: mount__default['default'], color: color__default['default'] });
  const dirs = typeof opts === 'string' ? [opts] : (opts.dirs || ['.']);
  dirs.forEach(setupStatic(app, opts));
  if (opts.proxy) Object.entries(opts.proxy).forEach(setupProxy(app));
  dirs.forEach(d => notice("serving", color.bold(path.resolve(d))));
  if (opts.spa) setupFallback(app, opts);
  if (!testing) {
    const server = app.listen({ port: (opts.port || 8080), host: opts.host }, () => printListenInfo(server));
  }
}

var index = (opts = {}) => ({
  name: 'dev-server',
  writeBundle() {
    if (!app) initApp.bind(this)(opts);
  }
});

module.exports = index;
